# coding: utf-8
from django.conf.urls.defaults import patterns

def get_dynamic_urls():
    from urls import dynamic_patterns
    from dynamic_pages.urls import DEFAULT_DYNAMIC_URLS
    try:
        dynamic_urls = list(dynamic_patterns)
        for default_pattern in DEFAULT_DYNAMIC_URLS:
            if not default_pattern.name in [pattern.name for pattern in dynamic_urls]:
                dynamic_urls.append(default_pattern)
        return dynamic_urls
    except:
        return []
    
def get_dynamic_url(name):
    for dynamic_url in get_dynamic_urls():
        if (dynamic_url.name == name):
            return dynamic_url
    return None
        
def get_dynamic_url_by_choice(choice):
    name = choice.split('-')[1]
    return get_dynamic_url(name)

def get_dynamic_url_choices():
    choices = []
    for dynamic_url in get_dynamic_urls():
        choices.append(dynamic_url.get_choice())
    return choices   

def dynamic_urlpatterns():
    from dynamic_pages.models import Page
    urlpatterns = patterns('')
    for page in Page.objects.all():
        if(page.pattern()):
            urlpatterns += page.pattern()
    return urlpatterns


# coding: utf-8
import re

from django.contrib.admin.util import quote as django_quote, unquote as django_unquote
from django.utils.encoding import force_unicode

def quote(url):
    url = django_quote(url)
    return re.sub(r' ', '__', url)

def unquote(url):
    url = re.sub(r'__', ' ', force_unicode(url))
    return django_unquote(url)


# coding: utf-8
import re

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

from utilities.models.fields import TreeForeignKey, PageTitleField, PageUrlField, HtmlField #@UnresolvedImport
       
META_TYPE = ( 
    ('description', u'description'), 
    ('keywords', u'keywords')
)
  
class Page(models.Model):
    updated = models.DateTimeField(_(u'Last modification'), auto_now=True)
    parent = TreeForeignKey('Page', verbose_name = _(u'Parent page'), null=True, blank=True)
    title = PageTitleField(_(u'Name'), max_length=255)
    relative_url = PageUrlField(_(u'URL'), max_length=100, blank=True, null=True)
    default = models.BooleanField(_(u'Main page'), default=False)
    order = models.IntegerField(_(u'Order'), null=True, blank=True, help_text=_(u'If you want add this page to the menu set the order'))
    page_type = models.CharField(_(u'Page type'), max_length=250)
    
    content = models.ForeignKey('PageContent',verbose_name = _(u'Page content'), null=True, blank=True)
    meta_data = models.ManyToManyField('Meta',verbose_name = _(u'Metadata'), null=True, blank=True)
    html_title = models.CharField(_(u'HTML Title'), max_length=255, null=True, blank=True)
    
    def __init__(self,  *args, **kwargs):
        super(Page, self).__init__(*args, **kwargs)
        from dynamic.utils import get_dynamic_url_choices
        self._meta.get_field_by_name('page_type')[0]._choices = list(get_dynamic_url_choices())
        
    def __unicode__(self):
        return self.title;
    
    def _parent_url(self):
        if (self.parent):
            return self.parent.url
        return ''
    parent_url = property(_parent_url)
        
    def _url(self):
        if (self.page_type_name == 'static'):
            return '/'+self.absolute_url
        if (self.page_type_name == 'redirects'):
            for qs in (Page.objects.filter(parent = self, order__isnull = False).order_by('order'), Page.objects.filter(parent = self, order__isnull = True)):
                for page in qs:
                    if (page.is_active() and page.is_in_menu()):
                        return page.url
        if (self.page_type_name == 'redirectstourl'):
            return self.content.cast().url
        if (self.page_type_name == 'redirectstopage'):
            return self.content.cast().page.url
        from templatetags.page_utils import dynamic_reverse 
        return dynamic_reverse(self.page_type_name)
    url = property(_url)
     
    def _absolute_url(self):
        if (self.parent):
            if not self.relative_url:
                return self.parent.absolute_url
            return self.parent.absolute_url+'/'+self.relative_url

        return self.relative_url
    
    absolute_url = property(_absolute_url)

    def pattern(self):
        from dynamic.utils import get_dynamic_url_by_choice
        dynamic_url = get_dynamic_url_by_choice(self.page_type)
        if (self.page_type_name != 'redirects'):
            return dynamic_url.get_url_patterns(self)
        return None
        
    def _page_type_name(self):
        return self.page_type.split('-')[1]
    
    page_type_name = property(_page_type_name)
            
    def save(self):
        from dynamic.utils import get_dynamic_url_by_choice
        url = get_dynamic_url_by_choice(self.page_type)
        if (not url.can_be_in_menu()):
            self.default = False
            self.order = None
        
        if(self.default):
            for page in Page.objects.filter(default=True):
                page.default = False
                super(Page, page).save()
            
        
        if (self.order == None): 
            super(Page, self).save()
        else:
            pages = Page.objects.filter(order__gte = self.order, parent = self.parent).order_by('order')
            first = True
            for page in pages:
                if (first and page.order != self.order):
                    break
                page.order += 1
                page.save()
                first = False
            super(Page, self).save()
        
    def delete(self):
        pages = Page.objects.filter(order__gt = self.order, parent = self.parent).order_by('order')
        for page in pages:
            page.order -= 1
            page.save()
        super(Page, self).delete()
        
    class Meta:
        verbose_name = _(u'Page')
        verbose_name_plural = _(u'Pages')
        ordering = ('order', )
    
    @staticmethod
    def get_page(path):
        from dynamic.utils import get_dynamic_url_by_choice
        path = re.sub(r'^/','', path)
        static_pages = Page.objects.exclude(page_type = 'none-redirects')
        for page in static_pages:
            dynamic_url = get_dynamic_url_by_choice(page.page_type)
            if dynamic_url.equals(page, path):
                return page
    
        return None
    
    def is_active(self):
        if (self.page_type_name != 'redirects'):
            return True
        for page in Page.objects.filter(parent = self):
            if (page.is_active()):
                return True
        return False
    
    def is_in_menu(self):
        return self.page_type.split('-')[2] == 'menu'   

        
class PageContent(models.Model):
    id = models.CharField(_(u'Id'), max_length=100, editable=False, primary_key=True)
    real_type = models.ForeignKey(ContentType, editable=False, null=True)
    
    def save(self, *args, **kwargs):
        if not self.id:
            self.real_type = self._get_real_type()
            model_name = '%s.%s' % (self._meta.app_label.lower(), self.__class__.__name__.lower())
            contents = self.__class__.objects.filter(Q(id__startswith='%s$' % model_name)).order_by('-id')
            if (contents):
                id = contents[0].id.split('$')[1]
                id = int(id) + 1
            else :
                id = 0
            self.id = '%s$%s' % (model_name, id)
        
        super(PageContent, self).save(*args, **kwargs)
    
    @staticmethod
    def get_content(path):
        page = Page.get_page(path)
        if (page and page.content):
            try:
                return PageContent.objects.get(pk = page.content.id).cast()
            except ObjectDoesNotExist:
                pass
        return None
    
    def _get_real_type(self):
        return ContentType.objects.get_for_model(type(self))

    def cast(self):
        return self.real_type.get_object_for_this_type(pk=self.pk)

    def __unicode__(self):
        return self.cast().__unicode__();
    
       
    class Meta:
        verbose_name = _(u'Page content')
        verbose_name_plural = _(u'Page content')


class DefaultPageContent(PageContent): 
    title = models.CharField(_(u'Title'), max_length=255) 
    subtitle = models.CharField(_(u'Subtitle'), max_length=255, null=True, blank=True)
    
    def __unicode__(self):
        return '%s - %s' % (unicode(_(u'Default page content')), self.title)
    
    class Meta:
        verbose_name = _(u'Default page content')
        verbose_name_plural = _(u'Default page content')

   
class RedirectToURLPageContent(PageContent):  
    url = models.URLField(_(u'URL address'), verify_exists=False)

    def __unicode__(self):
        return '%s - %s' % (unicode(_(u'Redirect to URL')), self.url);

    class Meta:
        verbose_name = _(u'Redirect to URL')
        verbose_name_plural = _(u'Redirect to URL')
        
class RedirectToPagePageContent(PageContent):
    page = TreeForeignKey(Page, verbose_name = _(u'Redirect to page'), parent='parent', limit_choices_to = ~models.Q(page_type__in = ['none-redirects-menu-true', 'dynamic_pages.redirecttourlpagecontent-redirectstourl-menu-false', 'dynamic_pages.redirecttopagepagecontent-redirectstopage-menu-false']))
                
    def __unicode__(self):
        return '%s - %s' % (unicode(_(u'Redirect to page')), self.page);

    class Meta:
        verbose_name = _(u'Redirect to page')
        verbose_name_plural = _(u'Redirect to page')
                  
class StaticPageContent(DefaultPageContent):
    html = HtmlField(_(u'HTML'), blank=True)   
   
    def __unicode__(self):
        return '%s - %s' % (unicode(_(u'Static content')), self.title);
    
    class Meta:
        verbose_name = _(u'Static content')
        verbose_name_plural = _(u'Static content')
        
        
class Meta(models.Model):
    name = models.CharField(_(u'Name'), max_length=50, choices=META_TYPE, default="description") 
    content = models.CharField(_(u'Text'), max_length=255, null=True, blank=True)
    
    def __unicode__(self):
        return '%s = %s' % (self.name, self.content);
       
    class Meta:
        verbose_name = _(u'HTML Meta tag')
        verbose_name_plural = _(u'HTML Meta tag')
        

class DynamicMeta(object):
    def __init__(self, name, content):
        self.name = name
        self.content = content



# coding: utf-8
from dynamic_pages.views import StaticView
from dynamic_pages.dynamic.dynamic_urls import DynamicUrl
from django.utils.translation import ugettext_lazy as _
    
DEFAULT_DYNAMIC_URLS = (
    DynamicUrl('static', _(u'Static page'),  StaticView.as_view(), (r'',), 'dynamic_pages.StaticPageContent'),
    DynamicUrl('redirects', _(u'Redirect to first child page')),
    DynamicUrl('redirectstourl', _(u'Redirect to URL'), None, None, 'dynamic_pages.RedirectToURLPageContent', can_change_url = False),
    DynamicUrl('redirectstopage', _(u'Redirect to page'), None, None, 'dynamic_pages.RedirectToPagePageContent', can_change_url = False),
)



# coding: utf-8
from django.contrib import admin
from django.contrib.admin.sites import AlreadyRegistered
from django.db.models import get_models

from utilities.admin import TreeModelMixin, RelatedToolsAdmin, HiddenModelAdmin

from form.page import PageForm
from models import Page, Meta, PageContent
    
class PageAdmin(TreeModelMixin, RelatedToolsAdmin):
    list_display = ('title','absolute_url','url', 'page_type', 'order')
    form = PageForm
    parent = 'parent'
    
   
    
    def get_obj(self, request):
        object_id = request.META['PATH_INFO'].strip('/').split('/')[-1]
        try:
            object_id = int(object_id)
        except ValueError:
            return None
        return Page.objects.get(pk=object_id)
    
    class Media:
        js = (
            '/static/dynamic_pages/js/jquery-1.6.4.min.js',
            '/static/dynamic_pages/js/pages.js'
        )

admin.site.register(Page, PageAdmin)            
admin.site.register(Meta, HiddenModelAdmin)   

def autoregister():
    '''
    Automatic registration PageContent models to admin
    '''
    for model in get_models():
        if(issubclass(model, PageContent)):
            try:
                admin.site.register(model, HiddenModelAdmin)
            except AlreadyRegistered:
                pass
autoregister()


# coding: utf-8
from django import forms
from django.utils.translation import ugettext as _

from dynamic_pages.models import Page #@UnresolvedImport
from dynamic_pages.dynamic.utils import get_dynamic_url_by_choice #@UnresolvedImport

class PageForm(forms.ModelForm):
    '''
    Dynamic page validation form
    '''
    
    def clean(self):
        cleaned_data = super(PageForm, self).clean()

        page_type = cleaned_data.get("page_type")
        if not self.instance.pk and (not (page_type in 'staticpagecontent-static-menu', 'redirecttourlpagecontent-redirectstourl-menu')) and Page.objects.filter(page_type = page_type):
            self._errors["page_type"] = self.error_class([_(u'Page with this type already exists')])
            

        content = cleaned_data.get("content")
        url = get_dynamic_url_by_choice(page_type)
        if (url.has_model()):
            if (not content):
                self._errors["content"] = self.error_class([_(u'Page content cannot be empty')])
            elif (url.get_model_name() != '%s.%s' % (content.cast()._meta.app_label, content.cast()._meta.object_name.lower())):
                self._errors["content"] = self.error_class([_(u'Wrong page content')])
        else:
            if (content):
                cleaned_data["content"] = None

        return cleaned_data
    
    class Meta:
        model = Page



# coding: utf-8
import re

from django.conf.urls.defaults import url

class DynamicUrl:
    '''
    Dynamic alternative to django url
    '''
    
    def __init__(self, name, verbose_name, view = None, patterns = [], model = None, can_change_url = True, view_kwargs = None):
        self.name = name
        self.view = view
        self.patterns = patterns
        self.model = model
        self.verbose_name = verbose_name
        self.can_change_url = can_change_url
        self.view_kwargs = view_kwargs
    
    def get_model_name(self):
        if not self.model:
            return 'none'
        return self.model.lower()
        
    def get_full_name(self):
        menu = 'nomenu'
        if (self.can_be_in_menu()):
            menu = 'menu'
        change_url = 'false'
        if (self.can_change_url):
            change_url = 'true'
        return '%s-%s-%s-%s' % (self.get_model_name(), self.name, menu, change_url)  
     
    def get_patterns(self, page):
        
        patterns = []
        if (not self.patterns):
            return patterns
        
        for pattern in self.patterns:
            pattern_parts = []
               
            if page.absolute_url != '':
                pattern_parts.append(page.absolute_url)
            
            if pattern != '':
                pattern_parts.append(pattern)
                
            patterns.append(r'^%s/?$'% '/'.join(pattern_parts))
        
        if (page.default):
            patterns.append(r'^/?$')
                
        return patterns
        
    def get_url_patterns(self, page):
        urls = []
        for pattern in self.get_patterns(page):
            urls.append(url(pattern, self.view, kwargs=self.view_kwargs, name=self.name))
        return urls  
           
    def can_be_in_menu(self):
        if (self.name in ('static', 'redirects', 'redirectstourl', 'redirectstopage')):
            return True
        
        for url_pattern in self.patterns:
            if (not re.search('\(.*[\?\|\+\.\*\[\]]+.*\)', url_pattern)):
                return True
        return False
       
    def get_choice(self):
        return (self.get_full_name(), self.verbose_name)
    
       
    def has_model(self):
        return self.model != None
    
    def equals(self, page, path):
        for pattern in self.get_patterns(page):
            if (re.search(pattern, path)):
                return True
        return False


# coding: utf-8
import sys

from datetime import datetime
from django.utils import translation
from django.conf import settings

from dynamic_pages.models import Page

class UrlsReloadMiddleware(object):
    '''
    Automatically updates urls.py
    '''
    
    last_reload = None
    count_pages = 0
    
    def process_request(self, request):
        if not self.last_reload or Page.objects.all().count() != self.count_pages or Page.objects.filter(updated__gte = self.last_reload):
            urlconf = settings.ROOT_URLCONF
            if urlconf in sys.modules:
                reload(sys.modules[urlconf])
        
            self.count_pages = Page.objects.all().count()
            self.last_reload = datetime.now()

        


# coding: utf-8
from django import template
from django.core.urlresolvers import reverse
from django.utils.encoding import smart_unicode

from dynamic_pages.dynamic.utils import get_dynamic_url
from dynamic_pages.utils import quote
from dynamic_pages.models import Page #@UnresolvedImport

register = template.Library()

@register.simple_tag
def is_active(link, req_path):
    current = Page.get_page(req_path)
    while(current != None):
        if (current == link):
            return "active"
        current = current.parent
    return "no-active"
    
def mainmenu(path):
    pages = Page.objects.filter(parent__isnull = True, order__isnull = False).order_by('order')
    for page in pages:
        if (not page.is_active()):
            pages = pages.exclude(pk = page.id)
    return {'pages': pages, 'path': path}
register.inclusion_tag('pages/menu/mainmenu.html')(mainmenu) 

def footmenu(path):
    pages = Page.objects.filter(parent__isnull = True, order__isnull = False).order_by('order')
    for page in pages:
        if (not page.is_active()):
            pages = pages.exclude(pk = page.id)
    return {'pages': pages, 'path': path}
register.inclusion_tag('pages/menu/footmenu.html')(footmenu) 


def submenu(path):
    parent = Page.get_page(path)
    if (parent):
        while(parent.parent != None):
            parent = parent.parent
        pages = Page.objects.filter(parent = parent, order__isnull = False).order_by('order')
        for page in pages:
            if (not page.is_active()):
                pages = pages.exclude(pk = page.id)
        return {'pages': pages, 'path': path}
    else:
        return
register.inclusion_tag('pages/menu/submenu.html')(submenu) 

def subsubmenu(context, path, form = None):
    if not form and context.has_key('form'):
        form = context['form']
    parent = Page.get_page(path)
    if (parent and parent.parent):
        while(parent.parent.parent != None):
            parent = parent.parent
        pages = Page.objects.filter(parent = parent, order__isnull = False).order_by('order')
        for page in pages:
            if (not page.is_active()):
                pages = pages.exclude(pk = page.id)
        return {'pages': pages, 'path': path, 'form': form, 'STATIC_URL': context['STATIC_URL']}
    else:
        return {'form': form, 'STATIC_URL': context['STATIC_URL']}
register.inclusion_tag('pages/menu/subsubmenu.html', takes_context=True)(subsubmenu) 

@register.simple_tag
def title(path, dynamic_title = ''):
    page = Page.get_page(path)
    if (page == None): return dynamic_title;
    
    title = page.title
    if (page.html_title != '' and page.html_title != None): title = page.html_title
    
    if (dynamic_title):
        return '%s - %s' % (title, dynamic_title)
    return title

def meta(path, meta = []):
    page = Page.get_page(path)
    if (page == None): return {'meta_tags': None}
    meta_tags = list(page.meta_data.all())
    meta_tags += meta
    return {'meta_tags': meta_tags}
register.inclusion_tag('pages/meta.html')(meta) 

@register.filter
def url_quote(value):
    return quote(value.encode("utf-8"))

import re


def dynamic_reverse(view_name, *args):
    url = get_dynamic_url(view_name)
    if not url:
        return ''
    
    pages = Page.objects.filter(page_type = url.get_full_name())
    if not pages:
        return ''
     
    page = pages[0]
    for pattern in url.get_patterns(page):
        if (not re.search('\(.*[\?\|\+\.\*\[\]]+.*\)', pattern)):
            pattern = re.sub(r'[\)\(]', '', pattern)
        
        if (re.search(r"((\([^\)]*\)[^\)\(]*)){%s}" % len(args), pattern)):
            reverse = ''
            
            for arg in args:
                pattern = re.sub('(\([^)]*\))', smart_unicode(arg), pattern, count=1)
            
            pattern = re.sub('[\?\^\$]', '', pattern) 
            if pattern == '/':
                return pattern
            return '/%s' % pattern 
        
    return ''
    
class DynamicReverseNode(template.Node):
    '''
    Dynamic alternative to django ReverseNode
    '''
    
    def __init__(self, view_name, args):
        self.view_name = str(view_name)
        self.args = args

    def render(self, context):
        args = [arg.resolve(context) for arg in self.args]
        return dynamic_reverse(self.view_name, *args)
    
@register.tag
def dynamicurl(parser, token):
    content = token.split_contents()
    args = []
    for arg in content:
        try:
            args.append(parser.compile_filter(arg))
        except:
            pass
    return DynamicReverseNode(args[1], args[2:])

class QueryNode(template.Node):
    '''
    Helps with updating and changing URL queryes in templates
    '''
    
    def __init__(self, data):
        self.data = data

    def render(self, context):
        params = context['request'].GET.copy()
        for del_params in self.data['remove']:
            if params.has_key(del_params):
                del params[del_params]
        
        for key, value in self.data['new_params'].items():
            params[key] = value.resolve(context)
        return params.urlencode()
    
@register.tag
def get_query_string(parser, token):
    content = token.split_contents()
    args = {
        'new_params': {},
        'remove': []    
        }
    for arg in content[1:]:
        type, values = arg.split(':')
        for value in values.split(','):
            try:
                if type == 'new_params':
                    param = value.split('=')
                    args[type][param[0]] = parser.compile_filter(param[1])
                elif type == 'remove':
                    args[type].append(value)
            except:
                pass

    return QueryNode(args)



from setuptools import setup, find_packages
from dynamic_pages import version

setup(
    name='django-dynamicpages',
    version=version,
    description="Application which allows dynamically change urls.",
    keywords='django, admin, pages',
    author='Lubos Matl',
    author_email='matllubos@gmail.com',
    url='https://github.com/matllubos/django-dynamicpages',
    license='GPL',
    package_dir={'dynamic_pages': 'dynamic_pages'},
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: Czech',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
    zip_safe=False
)